/// <reference types="node" />
import Task from '@dojo/core/async/Task';
import Tunnel, { TunnelProperties, DownloadOptions, ChildExecutor, NormalizedEnvironment } from './Tunnel';
import { Url } from 'url';
import { JobState } from './interfaces';
/**
 * A BrowserStack tunnel.
 *
 * The accessKey and username properties will be initialized using BROWSERSTACK_ACCESS_KEY and BROWSERSTACK_USERNAME.
 */
export default class BrowserStackTunnel extends Tunnel {
    /**
     * Whether or not to start the tunnel with only WebDriver support. Setting this value to `false` is not
     * supported.
     */
    automateOnly: boolean;
    /** If true, any other tunnels running on the account will be killed when the tunnel is started. */
    killOtherTunnels: boolean;
    /**
     * A list of server URLs that should be proxied by the tunnel. Only the hostname, port, and protocol are used.
     */
    servers: (Url | string)[];
    /** Skip verification that the proxied servers are online and responding at the time the tunnel starts. */
    skipServerValidation: boolean;
    /** If true, route all traffic via the local machine. */
    forceLocal: boolean;
    constructor(options?: BrowserStackOptions);
    readonly auth: string;
    readonly executable: string;
    readonly extraCapabilities: Object;
    readonly url: string;
    protected _postDownloadFile(data: Buffer, options?: DownloadOptions): Promise<void>;
    protected _makeArgs(...values: string[]): string[];
    sendJobState(jobId: string, data: JobState): Task<void>;
    protected _start(executor: ChildExecutor): Task<any>;
    protected _stop(): Promise<number>;
    /**
     * Attempt to normalize a BrowserStack described environment with the standard Selenium capabilities
     *
     * BrowserStack returns a list of environments that looks like:
     *
     * {
     *     "browser": "opera",
     *     "os_version": "Lion",
     *     "browser_version":"12.15",
     *     "device": null,
     *     "os": "OS X"
     * }
     *
     * @param environment a BrowserStack environment descriptor
     * @returns a normalized descriptor
     */
    protected _normalizeEnvironment(environment: any): NormalizedEnvironment;
}
export interface BrowserStackProperties extends TunnelProperties {
    automateOnly: boolean;
    killOtherTunnels: boolean;
    servers: (Url | string)[];
    skipServerValidation: boolean;
    forceLocal: boolean;
    environmentUrl: string;
}
export declare type BrowserStackOptions = Partial<BrowserStackProperties>;
