import Tunnel, { ChildExecutor, NormalizedEnvironment, TunnelProperties } from './Tunnel';
import { JobState } from './interfaces';
import Task from '@dojo/core/async/Task';
/**
 * A CrossBrowserTesting tunnel.
 *
 * This tunnel requires some non-standard configuration options (vs the other tunnels):
 *
 *   1. The capabilities must include the username, API key, browser_api_name, and os_api_name properties
 *   2. The Intern proxyUrl must use 'local' instead of 'localhost'
 *
 * An Intern config using this tunnel might be look like:
 *
 * ```js
 * define({
 * 	proxyUrl: 'http://local:9000',
 *
 * 	tunnel: 'CrossBrowserTesting',
 *
 * 	environments: [
 * 		{
 * 			browserName: 'chrome',
 * 			os_api_name: 'Win10',
 * 			browser_api_name: 'Chrome52'
 * 		}
 * 	]
 *
 * 	// Other Intern config options...
 * });
 * ```
 *
 * The username and accessKey properties will be initialized using CBT_USERNAME and CBT_APIKEY.
 */
export default class CrossBrowserTestingTunnel extends Tunnel {
    cbtVersion: string;
    readonly auth: string;
    readonly extraCapabilities: {
        username: string;
        password: string;
    };
    readonly isDownloaded: boolean;
    constructor(options?: TunnelProperties);
    download(forceDownload?: boolean): Task<any>;
    protected _makeArgs(readyFile: string): string[];
    sendJobState(jobId: string, data: JobState): Task<void>;
    protected _start(executor: ChildExecutor): Task<any>;
    /**
     * Attempt to normalize a TestingBot described environment with the standard Selenium capabilities
     *
     * TestingBot returns a list of environments that looks like:
     *
     * {
     *     "selenium_name": "Chrome36",
     *     "name": "googlechrome",
     *     "platform": "CAPITAN",
     *     "version":"36"
     * }
     *
     * @param environment a TestingBot environment descriptor
     * @returns a normalized descriptor
     */
    protected _normalizeEnvironment(environment: any): NormalizedEnvironment;
}
