"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Tunnel_1 = require("./Tunnel");
var util_1 = require("util");
var path_1 = require("path");
var Task_1 = require("@dojo/core/async/Task");
var util_2 = require("./util");
var lang_1 = require("@dojo/core/lang");
var SeleniumVersion = '3.3.1';
var ChromeVersion = '2.28';
var FirefoxVersion = '0.15.0';
var IEVersion = '3.3.0';
/**
 * A Selenium tunnel. This tunnel downloads the {@link http://www.seleniumhq.org/download/ Selenium-standalone server}
 * and any necessary WebDriver executables, and handles starting and stopping Selenium.
 *
 * The primary configuration option is {@linkcode module:digdug/SeleniumTunnel#drivers drivers}, which determines which
 * browsers the Selenium tunnel will support.
 *
 * Note that Java must be installed and in the system path to use this tunnel.
 */
var SeleniumTunnel = (function (_super) {
    __extends(SeleniumTunnel, _super);
    function SeleniumTunnel(options) {
        return _super.call(this, lang_1.mixin({
            seleniumArgs: null,
            drivers: ['chrome'],
            baseUrl: 'https://selenium-release.storage.googleapis.com',
            version: SeleniumVersion,
            seleniumTimeout: 5000
        }, options)) || this;
    }
    Object.defineProperty(SeleniumTunnel.prototype, "artifact", {
        get: function () {
            return "selenium-server-standalone-" + this.version + ".jar";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SeleniumTunnel.prototype, "directory", {
        get: function () {
            return path_1.join(__dirname, 'selenium-standalone');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SeleniumTunnel.prototype, "executable", {
        get: function () {
            return 'java';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SeleniumTunnel.prototype, "isDownloaded", {
        get: function () {
            var directory = this.directory;
            return util_2.fileExists(path_1.join(directory, this.artifact)) && this._getDriverConfigs().every(function (config) {
                return util_2.fileExists(path_1.join(directory, config.executable));
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SeleniumTunnel.prototype, "url", {
        get: function () {
            var majorMinorVersion = this.version.slice(0, this.version.lastIndexOf('.'));
            return util_1.format('%s/%s/%s', this.baseUrl, majorMinorVersion, this.artifact);
        },
        enumerable: true,
        configurable: true
    });
    SeleniumTunnel.prototype.download = function (forceDownload) {
        var _this = this;
        if (forceDownload === void 0) { forceDownload = false; }
        if (!forceDownload && this.isDownloaded) {
            return Task_1.default.resolve();
        }
        var tasks;
        return new Task_1.default(function (resolve, reject) {
            var configs = [
                { url: _this.url, executable: _this.artifact }
            ].concat(_this._getDriverConfigs());
            tasks = configs.map(function (config) {
                var executable = config.executable;
                if (util_2.fileExists(path_1.join(_this.directory, executable))) {
                    return Task_1.default.resolve();
                }
                // TODO: progress events
                return _this._downloadFile(config.url, _this.proxy, {
                    executable: executable
                });
            });
            resolve(Promise.all(tasks));
        }, function () {
            tasks && tasks.forEach(function (task) {
                task.cancel();
            });
        });
    };
    SeleniumTunnel.prototype.sendJobState = function () {
        // This is a noop for Selenium
        return Task_1.default.resolve();
    };
    SeleniumTunnel.prototype._getDriverConfigs = function () {
        function getDriverConfig(name, options) {
            var Constructor = driverNameMap[name];
            if (!Constructor) {
                throw new Error('Invalid driver name "' + name + '"');
            }
            return new Constructor(options);
        }
        return this.drivers.map(function (data) {
            if (typeof data === 'string') {
                return getDriverConfig(data);
            }
            if (typeof data === 'object' && data.name) {
                return getDriverConfig(data.name, data);
            }
            // data is a driver definition
            return data;
        });
    };
    SeleniumTunnel.prototype._makeArgs = function () {
        var directory = this.directory;
        var driverConfigs = this._getDriverConfigs();
        var args = [];
        driverConfigs.reduce(function (args, config) {
            var file = path_1.join(directory, config.executable);
            args.push('-D' + config.seleniumProperty + '=' + file);
            return args;
        }, args);
        if (this.seleniumArgs) {
            args.push.apply(args, this.seleniumArgs);
        }
        args.push('-jar', path_1.join(this.directory, this.artifact), '-port', this.port);
        if (this.verbose) {
            args.push('-debug');
            console.log('Starting with arguments: ', args.join(' '));
        }
        return args;
    };
    SeleniumTunnel.prototype._postDownloadFile = function (data, options) {
        if (path_1.extname(options.executable) === '.jar') {
            return util_2.writeFile(data, path_1.join(this.directory, options.executable));
        }
        return _super.prototype._postDownloadFile.call(this, data, options);
    };
    SeleniumTunnel.prototype._start = function (executor) {
        var _this = this;
        var handle;
        var task = this._makeChild(function (child, resolve, reject) {
            handle = util_2.on(child.stderr, 'data', function (data) {
                // Selenium recommends that we poll the hub looking for a status response
                // https://github.com/seleniumhq/selenium-google-code-issue-archive/issues/7957
                // We're going against the recommendation here for a few reasons
                // 1. There's no default pid or log to look for errors to provide a specific failure
                // 2. Polling on a failed server start could leave us with an unpleasant wait
                // 3. Just polling a selenium server doesn't guarantee it's the server we started
                // 4. This works pretty well
                data = String(data);
                if (data.indexOf('Selenium Server is up and running') > -1) {
                    resolve();
                }
                else if (data.indexOf('Address already in use') !== -1) {
                    reject(new Error('Address is already in use'));
                    // Kill the child since we're reporting that startup failed
                    child.kill('SIGINT');
                }
                if (_this.verbose) {
                    process.stderr.write(data);
                }
            });
            executor(child, resolve, reject);
        });
        task.then(handle.destroy.bind(handle), handle.destroy.bind(handle));
        return task;
    };
    return SeleniumTunnel;
}(Tunnel_1.default));
exports.default = SeleniumTunnel;
var Config = (function () {
    function Config(config) {
        lang_1.mixin(this, config);
    }
    return Config;
}());
var ChromeConfig = (function (_super) {
    __extends(ChromeConfig, _super);
    function ChromeConfig() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.arch = process.arch;
        _this.baseUrl = 'https://chromedriver.storage.googleapis.com';
        _this.platform = process.platform;
        _this.version = ChromeVersion;
        return _this;
    }
    Object.defineProperty(ChromeConfig.prototype, "artifact", {
        get: function () {
            var platform = this.platform;
            if (platform === 'linux') {
                platform = 'linux' + (this.arch === 'x64' ? '64' : '32');
            }
            else if (platform === 'darwin') {
                var parts_1 = String(this.version).split('.').map(Number);
                var isGreater = [2, 22].some(function (part, i) {
                    return parts_1[i] > part;
                });
                platform = isGreater ? 'mac64' : 'mac32';
            }
            return util_1.format('chromedriver_%s.zip', platform);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ChromeConfig.prototype, "url", {
        get: function () {
            return util_1.format('%s/%s/%s', this.baseUrl, this.version, this.artifact);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ChromeConfig.prototype, "executable", {
        get: function () {
            return this.platform === 'win32' ? 'chromedriver.exe' : 'chromedriver';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ChromeConfig.prototype, "seleniumProperty", {
        get: function () {
            return 'webdriver.chrome.driver';
        },
        enumerable: true,
        configurable: true
    });
    return ChromeConfig;
}(Config));
var FirefoxConfig = (function (_super) {
    __extends(FirefoxConfig, _super);
    function FirefoxConfig() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.arch = process.arch;
        _this.baseUrl = 'https://github.com/mozilla/geckodriver/releases/download';
        _this.platform = process.platform;
        _this.version = FirefoxVersion;
        return _this;
    }
    Object.defineProperty(FirefoxConfig.prototype, "artifact", {
        get: function () {
            var platform = this.platform;
            if (platform === 'linux') {
                platform = 'linux' + (this.arch === 'x64' ? '64' : '32');
            }
            else if (platform === 'win32') {
                platform = 'win' + (this.arch === 'x64' ? '64' : '32');
            }
            else if (platform === 'darwin') {
                platform = 'macos';
            }
            var extension = /^win/.test(platform) ? '.zip' : '.tar.gz';
            return util_1.format('geckodriver-v%s-%s%s', this.version, platform, extension);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FirefoxConfig.prototype, "url", {
        get: function () {
            return util_1.format('%s/v%s/%s', this.baseUrl, this.version, this.artifact);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FirefoxConfig.prototype, "executable", {
        get: function () {
            return this.platform === 'win32' ? 'geckodriver.exe' : 'geckodriver';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(FirefoxConfig.prototype, "seleniumProperty", {
        get: function () {
            return 'webdriver.gecko.driver';
        },
        enumerable: true,
        configurable: true
    });
    return FirefoxConfig;
}(Config));
var IEConfig = (function (_super) {
    __extends(IEConfig, _super);
    function IEConfig() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.arch = process.arch;
        _this.baseUrl = 'https://selenium-release.storage.googleapis.com';
        _this.version = IEVersion;
        return _this;
    }
    Object.defineProperty(IEConfig.prototype, "artifact", {
        get: function () {
            var architecture = this.arch === 'x64' ? 'x64' : 'Win32';
            return util_1.format('IEDriverServer_%s_%s.zip', architecture, this.version);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IEConfig.prototype, "url", {
        get: function () {
            var majorMinorVersion = this.version.slice(0, this.version.lastIndexOf('.'));
            return util_1.format('%s/%s/%s', this.baseUrl, majorMinorVersion, this.artifact);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IEConfig.prototype, "executable", {
        get: function () {
            return 'IEDriverServer.exe';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IEConfig.prototype, "seleniumProperty", {
        get: function () {
            return 'webdriver.ie.driver';
        },
        enumerable: true,
        configurable: true
    });
    return IEConfig;
}(Config));
var driverNameMap = {
    chrome: ChromeConfig,
    firefox: FirefoxConfig,
    ie: IEConfig
};
//# sourceMappingURL=SeleniumTunnel.js.map