import Tunnel, { TunnelProperties, ChildExecutor, NormalizedEnvironment } from './Tunnel';
import { JobState } from './interfaces';
import Task from '@dojo/core/async/Task';
/**
 * A TestingBot tunnel.
 *
 * The username and accessKey properties will be initialized using TESTINGBOT_API_KEY and TESTINGBOT_API_SECRET.
 */
export default class TestingBotTunnel extends Tunnel implements TunnelProperties {
    /**
     * A list of regular expressions corresponding to domains whose connections should fail immediately if the VM
     * attempts to make a connection to them.
     */
    fastFailDomains: string[];
    /** A filename where additional logs from the tunnel should be output. */
    logFile: string;
    /** Whether or not to use rabbIT compression for the tunnel connection. */
    useCompression: boolean;
    /** Whether or not to use the default local Jetty proxy for the tunnel. */
    useJettyProxy: boolean;
    /** Whether or not to use the default remote Squid proxy for the VM. */
    useSquidProxy: boolean;
    /** Whether or not to re-encrypt data encrypted by self-signed certificates. */
    useSsl: boolean;
    constructor(options?: TestingBotOptions);
    readonly auth: string;
    readonly isDownloaded: boolean;
    protected _makeArgs(readyFile: string): string[];
    sendJobState(jobId: string, data: JobState): Task<void>;
    protected _start(executor: ChildExecutor): Task<any>;
    /**
     * Attempt to normalize a TestingBot described environment with the standard Selenium capabilities
     *
     * TestingBot returns a list of environments that looks like:
     *
     * {
     *     "selenium_name": "Chrome36",
     *     "name": "googlechrome",
     *     "platform": "CAPITAN",
     *     "version":"36"
     * }
     *
     * @param environment a TestingBot environment descriptor
     * @returns a normalized descriptor
     */
    protected _normalizeEnvironment(environment: any): NormalizedEnvironment;
}
export interface TestingBotProperties extends TunnelProperties {
    fastFailDomains: string[];
    logFile: string;
    useCompression: boolean;
    useJettyProxy: boolean;
    useSquidProxy: boolean;
    useSsl: boolean;
}
export declare type TestingBotOptions = Partial<TestingBotProperties>;
